import * as React from 'react';
import RefractorCore from 'react-refractor/all';
import clsx from 'clsx';

type PrismDerivedElement = HTMLDivElement;

export interface PrismProps extends Omit<React.HTMLProps<PrismDerivedElement>, 'children'> {
  code?: string;
  language?: string;
  lineNumbers?: boolean;
  tabSize?: number;
  header?: boolean;
}

export const Refractor = React.forwardRef<PrismDerivedElement, PrismProps>(({
  code = '',
  language = 'plain',
  lineNumbers = false,
  tabSize = 2,
  className,
  style,
  header = false,
  ...etcProps
}, forwardedRef) => {
  return (
    <div
      className={clsx(
        'react-refractor',
        className,
      )}
      style={style}
      ref={forwardedRef}
    >
      {header && (
        <div className="flex items-center justify-between">
          <div className="uppercase text-sm">
            {language}
          </div>
          <form>
            <button
              type="submit"
              name="action"
              value="copyContents"
              className="h-12 uppercase text-primary font-bold"
            >
              Copy
            </button>
          </form>
        </div>
      )}
      <div
        className="flex gap-2 items-start"
      >
        {lineNumbers && (
          <pre
            style={{
              textAlign: 'right',
            }}
          >
            <code>
              {code.split('\n').map((_, i) => (
                <React.Fragment key={i}>
                  {i > 0 && '\n'}
                  {i + 1}
                </React.Fragment>
              ))}
            </code>
          </pre>
        )}
        <div className="flex-auto overflow-auto">
          <RefractorCore
            {...etcProps}
            language={language}
            value={code}
            className="highlight"
          />
        </div>
      </div>
    </div>
  )
});

Refractor.displayName = 'Refractor';

